from typing import TypeVar, Any
from concurrent.futures import Future

T = TypeVar("T")

def f_return(x: T) -> Future[T]: ...
def f_return_error(x: BaseException, traceback: Any = ...) -> Future[None]: ...
def f_return_cancelled() -> Future[None]: ...
